-----------------------------------------------------------------------------------------------
--[[
HousingRepairAlerts
    Launches when you zone in to your house. Has options to report plots that need repair and
    plots that will need repair in a configurable amount of time. Reports can be a popup when
    you first enter your house zone, a message in the chat window, or both.

    Can be configured to ignore certain plots, since you probably don't want to repair that 
    Cute Date Spot (it does nothing special when broken).

    No localization, but a lot of the display text is taken from the game, so it may be 
    partially understandable by non-English users.

    TODO: when something needs repair, make the popup window a button that calls Event_FireGenericEvent("HousingButtonLandscape")
    TODO: Put LaunchLandscapeOnBubbleClick in options dialog

--]]
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "HousingLib"
require "ChatSystemLib" 

-----------------------------------------------------------------------------------------------
-- RepairAlerts Module Definition
-----------------------------------------------------------------------------------------------
local RepairAlerts = {} 

-----------------------------------------------------------------------------------------------
-- String literals
-----------------------------------------------------------------------------------------------
local kAddonTitle = "Housing Repair Alerts"

local kMaxIgnore = 6
local kControls = 
{
    ShowInBubble           = "BtnShowAlert",
    ShowUpcomingInBubble   = "BtnAlertIncludeUpcoming",
    ShowInChat             = "BtnShowChat",
    ShowUpcomingInChat     = "BtnChatIncludeUpcoming",

    LandscapeOnBubbleClick = "BtnClickToRepair",

    SliderBubbleDuration   = "SliderBubbleDuration",
    SliderUpcomingDuration = "SliderUpcomingDuration",
    EditBubbleDuration     = "EditBubbleDuration",
    EditUpcomingDuration   = "EditUpcomingDuration",

    BubbleText             = "RepairAlertsBubbleText",

    FrameIgnorePlots       = "FrameIgnorePlots",

    IgnoreCheckPrefix      = "BtnIgnore", -- This has the numbers 1 to 6 appended to get Button identifiers
}

local kDefault = 
{
    BubbleDurationMin   = 1,
    BubbleDuration      = 5, -- Seconds
    BubbleDurationMax   = 60,

    UpcomingDurationMin = 1,
    UpcomingDuration    = 24, -- Hours
    UpcomingDurationMax = 100,
}

-- Formatting strings for output
--TODO: Consolidate newline, spacing items, and parens somewhere
local kTextFormat = 
{
    Chat = 
    {
        NeedsRepair    = {HeaderText = "Housing Repairs Overdue:",},
        UpcomingRepair = {HeaderText = "Housing Repairs Coming Up Soon:",},
    },
    Bubble = 
    {
        Spacer = "<P TextColor=\"00ffffff\" Font=\"CRB_InterfaceTiny\">.</P>", -- Zero opacity
        
        NeedsRepair = 
        {
            HeaderText = "Housing Repairs Overdue:",
            Header = 
            {
                Font      = "CRB_Header20",
                TextColor = "white", 
                Alignment = "Left",
                GetWidth  = function(t, str) return Apollo.GetTextWidth(t.Font, str) end,
                GetAML    = function(t, str) return string.format("<P Align=\"%s\" TextColor=\"%s\" Font=\"%s\">%s</P>", t.Alignment, t.TextColor, t.Font, str) end
            },
            PlotName = 
            {
                Font      = "CRB_Interface16",
                Icon      = "CRB_BreakoutSprites:spr_BreakoutStun_ClockGreen", -- Yes, "ClockGreen" is red
                TextColor = "UI_TextHoloTitle",
                Alignment = "Left",
                GetWidth  = function(t, str) return Apollo.GetTextWidth(t.Font, str) end,
                GetAML    = function(t, str) return string.format("<P Align=\"%s\" TextColor=\"%s\" Font=\"%s\"><T Image=\"%s\"></T>%s</P>", t.Alignment, t.TextColor, t.Font, t.Icon, str) end
            }
        },
        UpcomingRepair = 
        {
            HeaderText = "Housing Repairs Coming Up Soon:",
            Header = 
            {
                Font      = "CRB_Header20",
                TextColor = "white",
                Alignment = "Left",
                GetWidth  = function(t, str) return Apollo.GetTextWidth(t.Font, str) end,
                GetAML    = function(t, str) return string.format("<P Align=\"%s\" TextColor=\"%s\" Font=\"%s\">%s</P>", t.Alignment, t.TextColor, t.Font, str) end
            },
            PlotName = 
            {
                Font      = "CRB_Interface16",
                Icon      = "CRB_BreakoutSprites:spr_BreakoutStun_ClockOrange", 
                TextColor = "UI_TextHoloTitle",
                Alignment = "Left",
                GetWidth  = function(t, str) return Apollo.GetTextWidth(t.Font, str) end,
                GetAML    = function(t, str) return string.format("<P Align=\"%s\" TextColor=\"%s\" Font=\"%s\"><T Image=\"%s\"></T>%s</P>", t.Alignment, t.TextColor, t.Font, t.Icon, str) end
            }
        }
    }
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function RepairAlerts:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    self.wndConfigure = nil
    self.wndBubble    = nil

    self.tOptions = 
    {
        UpcomingDuration       = kDefault.UpcomingDuration,
        BubbleDuration         = kDefault.BubbleDuration,
        ShowInBubble           = true,
        ShowUpcomingInBubble   = true,
        LandscapeOnBubbleClick = true,

        ShowInChat             = true,
        ShowUpcomingInChat     = true,


        IgnoredPlots = {}               -- Plot Ids to ignore. Some people leave things unrepaired on purpose
    }

    return o
end

function RepairAlerts:Init()
    local bHasConfigureFunction = true
    local strConfigureButtonText = kAddonTitle  --TODO: Is this too long to look good on the system escape menu?
    local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- RepairAlerts OnLoad
-----------------------------------------------------------------------------------------------
function RepairAlerts:OnLoad()
    -- load our form file
    self.xmlDoc = XmlDoc.CreateFromFile("HousingRepairAlerts.xml")
    self.xmlDoc:RegisterCallback("OnDocLoaded", self)
    
    Apollo.RegisterEventHandler("SubZoneChanged", "OnSubZoneChanged", self)	
end

-----------------------------------------------------------------------------------------------
-- RepairAlerts OnDocLoaded
-----------------------------------------------------------------------------------------------
function RepairAlerts:OnDocLoaded()
    if self.xmlDoc == nil       then return end -- Shouldn't happen, but the Carbine addons test for it
    if not self.xmlDoc.IsLoaded then return end -- Also shouldn't happen

        self.wndConfigure = Apollo.LoadForm(self.xmlDoc, "RepairAlertsConfigureForm", nil, self)
        if self.wndConfigure == nil then
            Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
            return
        end
        self.wndConfigure:FindChild(kControls.SliderBubbleDuration):SetMinMax  (kDefault.BubbleDurationMin,   kDefault.BubbleDurationMax)
        self.wndConfigure:FindChild(kControls.SliderUpcomingDuration):SetMinMax(kDefault.UpcomingDurationMin, kDefault.UpcomingDurationMax)

        self.wndConfigure:Show(false, true)

        self.wndBubble = Apollo.LoadForm(self.xmlDoc, "RepairAlertsBubble", nil, self)
        if self.wndBubble == nil then
            Apollo.AddAddonErrorText(self, "Could not load the popup window for some reason.")
            return
        end

        self.xmlDoc = nil
        
        --Let's be kind and handle both singular and plural: "repairalert" and "repairalerts"
        Apollo.RegisterSlashCommand("repairalerts", "OnSlashRepairAlerts", self)
        Apollo.RegisterSlashCommand("repairalert",  "OnSlashRepairAlerts", self)
end

-----------------------------------------------------------------------------------------------
-- State persistence
-----------------------------------------------------------------------------------------------
function RepairAlerts:OnSave(eLevel)
    if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then return {SaveVersion = 2, SaveData = self.tOptions} end
end

function RepairAlerts:OnRestore(eLevel, tData)
    if eLevel == GameLib.CodeEnumAddonSaveLevel.Character and tData ~= nil then
        if tData.SaveVersion == 1 or tData.SaveVersion == 2 then
            if tData.SaveData.UpcomingDuration       ~= nil then self.tOptions.UpcomingDuration       = tData.SaveData.UpcomingDuration;       end
            if tData.SaveData.BubbleDuration         ~= nil then self.tOptions.BubbleDuration         = tData.SaveData.BubbleDuration;         end
            if tData.SaveData.ShowInBubble           ~= nil then self.tOptions.ShowInBubble           = tData.SaveData.ShowInBubble;           end
            if tData.SaveData.ShowUpcomingInBubble   ~= nil then self.tOptions.ShowUpcomingInBubble   = tData.SaveData.ShowUpcomingInBubble;   end
            if tData.SaveData.ShowInChat             ~= nil then self.tOptions.ShowInChat             = tData.SaveData.ShowInChat;             end
            if tData.SaveData.ShowUpcomingInChat     ~= nil then self.tOptions.ShowUpcomingInChat     = tData.SaveData.ShowUpcomingInChat;     end
            if tData.SaveData.IgnoredPlots           ~= nil then self.tOptions.IgnoredPlots           = tData.SaveData.IgnoredPlots;           end
            if tData.SaveData.LandscapeOnBubbleClick ~= nil then self.tOptions.LandscapeOnBubbleClick = tData.SaveData.LandscapeOnBubbleClick; end
        else
            self:ErrorOut("Unrecognized format of saved configuration. Saved configuration has been ignored.")
        end
    end
end

-----------------------------------------------------------------------------------------------
-- OnConfigure
-----------------------------------------------------------------------------------------------
function RepairAlerts:OnConfigure()
    if self.wndConfigure == nil then return end

    self.wndConfigure:Show(true)
    self.wndConfigure:ToFront()
end

-----------------------------------------------------------------------------------------------
-- OnSubZoneChanged
-----------------------------------------------------------------------------------------------
function RepairAlerts:OnSubZoneChanged(idZone, strZoneName)
    if self.wndConfigure and self.wndConfigure:IsValid() and self.wndConfigure:IsVisible() then self.wndConfigure:Close(); end

    if not HousingLib.IsHousingWorld()  then return end
    if not HousingLib.IsOnMyResidence() then return end

    -- Okay, we're on our house. Get a list of plugs that need repair and one of plugs that 
    -- will need repair soon
    
    --TODO: Does IsOnMyResidence() return true for roommates? For warplots?
    
    local tPlotsNeedRepair     = {}
    local tPlotsNeedRepairSoon = {}
    
    local nPlots = HousingLib.GetPlotCount()
    for iPlot = 1, nPlots do
        local tPlotInfo = HousingLib.GetPlot(iPlot)
        
        if self:PlotNeedsRepair(tPlotInfo) then
            -- Gather list of plots that currently need repair
            table.insert(tPlotsNeedRepair, tPlotInfo)
            
        elseif self:PlotNeedsRepairSoon(tPlotInfo) then 
            -- Gather list of plots that will need repair in the watch window
            table.insert(tPlotsNeedRepairSoon, tPlotInfo)
        end
    end

    -- Moved responsibility for checking if Alerts and Chat are turned on to the formatting function, which is probably silly
    local strBubbleAML, nWidth = self:MakeBubbleAML(tPlotsNeedRepair, tPlotsNeedRepairSoon)
    if strBubbleAML then self:ShowBubble(strBubbleAML, nWidth) end

    local strChatText = self:MakeChatText(tPlotsNeedRepair, tPlotsNeedRepairSoon)
    if strChatText then self:MessageOut(strChatText) end
    
end

-----------------------------------------------------------------------------------------------
-- RepairAlerts Functions
-----------------------------------------------------------------------------------------------
-- on SlashCommand "/repairalert" or "/repairalerts"
function RepairAlerts:OnSlashRepairAlerts(strCmd, strParams)
    -- No params means to show the configuration dialog
    if strParams == nil or strParams == "" then  self.wndConfigure:Invoke(); return; end; -- show the window

    -- For testing, pretend we just zoned in
    if strParams == "test" then self:OnSubZoneChanged(1,"a") end
end

-----------------------------------------------------------------------------------------------
-- Internal functions
-----------------------------------------------------------------------------------------------
-- A plot needs repair if it is a) inactive, b) time-based, and c) out of time
function RepairAlerts:PlotNeedsRepair(tPlotInfo)
    return tPlotInfo.bActive         == false
       and tPlotInfo.eUpkeepType     == HousingLib.HousingUpkeepType.Timed
       and tPlotInfo.fRemainingHours == 0
       and self.tOptions.IgnoredPlots[tPlotInfo.nPlugItemId] ~= true
end

-- A plot will need repairs soon if it 
--  a) is active, 
--  b) is time-based, and 
--  c) has no more than the window remaining (but more than zero time)
function RepairAlerts:PlotNeedsRepairSoon(tPlotInfo)
    return tPlotInfo.bActive 		 == true
       and tPlotInfo.eUpkeepType 	 == HousingLib.HousingUpkeepType.Timed
       and tPlotInfo.fRemainingHours >  0
       and tPlotInfo.fRemainingHours <= self.tOptions.UpcomingDuration
       and self.tOptions.IgnoredPlots[tPlotInfo.nPlugItemId] ~= true
end

-- Turns a floating point "hours remaining" into the Carbine-approved text for time remaining. Largely cribbed from the Landscape screen
function RepairAlerts:TimeRemainingString(fRemainingHours)
    local Days    = math.floor(fRemainingHours/24)
    local Hours   = math.floor(fRemainingHours - Days*24)
    local Minutes = math.floor((fRemainingHours - Days*24 - Hours) * 60) -- Would this be clearer as "math.floor((fRemainingHours - math.floor(fRemainingHours)) * 60)"?
    
    -- Got these calls from the Landscpe window code. No idea what a WeaselString is. Nor why the last option doesn't need one.
    local strRet = nil
    if     Days    >  1 then                    strRet = String_GetWeaselString(Apollo.GetString("HousingLandscape_UpkeepDaysHours"),         Days,  Hours)
    elseif Minutes >= 1 and Minutes < 10 then	strRet = String_GetWeaselString(Apollo.GetString("HousingLandscape_UpkeepHoursMinsWithZero"), Hours, Minutes)
    elseif Minutes >= 1 then					strRet = String_GetWeaselString(Apollo.GetString("HousingLandscape_UpkeepHoursMins"),         Hours, Minutes)
    else 							            strRet = Apollo.GetString("HousingLandscape_UpkeepLessThanMin") -- Nope, no "weasel string" for this one
    end
    
    return strRet
end

-----------------------------------------------------------------------------------------------
-- Load up the alert bubble and show it
function RepairAlerts:ShowBubble(strAML, nWidth)
    if self.wndBubble == nil then return end
    if strAML == nil then return end

    -- Create a timer to close the window
    local oTimer = ApolloTimer.Create(self.tOptions.BubbleDuration, false, "OnBubbleTimeout", self) --TODO: replace the naked bool with an enum
    self.wndBubble:SetData(oTimer) -- We need to keep a reference to the timer around. If it gets GCd then the timer stops. Just stick it on the window.

    -- Now let's put the text in the window
    local wndBubbleText = self.wndBubble:FindChild(kControls.BubbleText)
    if wndBubbleText then
        wndBubbleText:SetAML(strAML)
        
        -- Now try to get the width needed for the text and set the MLWindow to that
        local nLeft, nTop, nRight, nBottom = self.wndBubble:GetAnchorOffsets()                  -- Containing window offsets
        if nWidth and nWidth > 100 then 
            local nBubbleWidthOffsetSize = (nWidth + 120) / 2                                   -- This seems to work, but I don't know why we need the 120
            nLeft  = 0 - nBubbleWidthOffsetSize                                                 -- Left is center of the screen minus half of (width plus magic padding) 
            nRight = 0 + nBubbleWidthOffsetSize                                                 -- Right is center of the screen plus half of (width plus magic padding)
            self.wndBubble:SetAnchorOffsets(nLeft, nTop, nRight, nBottom)                       -- Resize the containing window so the MLWindow can fit in it horizontally. ML resizes automatically.
            wndBubbleText:RecalculateContentExtents()                                           -- Ask the MLWindow to recalculate
        end   
        local nTextWidth, nTextHeight = wndBubbleText:SetHeightToContentHeight()                -- Ask the MLWindow to grow vertically
        self.wndBubble:SetAnchorOffsets(nLeft, nTop, nRight, nTop + nTextHeight + 110)           -- Resize the containing window with a hardcoded size increase from StoryPanel.lua

        wndBubbleText:Show(true)
        wndBubbleText:ToFront()

        self.wndBubble:Show(true)
        self.wndBubble:ToFront()
    end
end

function RepairAlerts:OnBubbleTimeout()
    self:HideBubble()
end

function RepairAlerts:OnRepairAlertsBubbleMouseDown(wndHandler, wndControl, eMouseButton, nX, nY, bDoubleClick)
    if self.tOptions.LandscapeOnBubbleClick == true then Event_FireGenericEvent("HousingButtonLandscape") end
    self:HideBubble()
    return true -- stop propagation (don't want to accidentally click through it)
end

function RepairAlerts:HideBubble()
    self.wndBubble:SetData(nil)  -- Lose the last reference to the timer so the GC stops it. We could be closing because the uset clicked Escape or clicked on the bubble. Might as well not also time out an invisible window.
    self.wndBubble:Close()
end

function RepairAlerts:OnCancelBubble(wndHandler, wndControl, eMouseButton)
    self:HideBubble()
end

function RepairAlerts:OnConfigureBubble(wndHandler, wndControl, eMouseButton)
    self.wndConfigure:Invoke()
end

---------------------------------------------------------------------------------------------------
-- Output formatting
---------------------------------------------------------------------------------------------------
-- This method returns the markup text as the first retval and the widest line width as the second
function RepairAlerts:MakeBubbleAML(tNeedsRepair, tUpcomingRepair)
    if self.tOptions.ShowInBubble ~= true then return nil end -- No AML to show if we don't show anything

    local str    = nil -- Accumulates return content
    local nWidth = 0

    if tNeedsRepair and #tNeedsRepair > 0 then
        -- Header line
        local strHeader = kTextFormat.Bubble.NeedsRepair.HeaderText
        local strNeedsRepair = kTextFormat.Bubble.NeedsRepair.Header:GetAML(strHeader)   
        nWidth = math.max(nWidth, kTextFormat.Bubble.NeedsRepair.Header:GetWidth(strHeader))
        
        -- One line per plug
        for i,v in ipairs(tNeedsRepair) do
            local strLine = v.strName
            strNeedsRepair = strNeedsRepair..kTextFormat.Bubble.NeedsRepair.PlotName:GetAML(strLine)
            nWidth = math.max(nWidth, kTextFormat.Bubble.NeedsRepair.PlotName:GetWidth(strLine))
        end
        if str == nil then str = strNeedsRepair else str = str..strNeedsRepair end
    end

    if tUpcomingRepair and #tUpcomingRepair > 0 and self.tOptions.ShowUpcomingInBubble then
        -- Header for upcoming repairs
        local strHeader = kTextFormat.Bubble.UpcomingRepair.HeaderText
        local strUpcomingRepair = kTextFormat.Bubble.UpcomingRepair.Header:GetAML(strHeader)  
        nWidth = math.max(nWidth, kTextFormat.Bubble.UpcomingRepair.Header:GetWidth(strHeader))

        -- One line per repair
        for i,v in ipairs(tUpcomingRepair) do
            local strLine = v.strName.." ("..self:TimeRemainingString(v.fRemainingHours)..")"
            strUpcomingRepair = strUpcomingRepair .. kTextFormat.Bubble.UpcomingRepair.PlotName:GetAML(strLine)
            nWidth = math.max(nWidth, kTextFormat.Bubble.UpcomingRepair.PlotName:GetWidth(strLine))
        end
        if str == nil then str = strUpcomingRepair else str = str..kTextFormat.Bubble.Spacer..strUpcomingRepair end
    end

    return str, nWidth
end


function RepairAlerts:MakeChatText(tNeedsRepair, tUpcomingRepair)
    if self.tOptions.ShowInChat ~= true then return nil end -- No text to show if we aren't showing any text 

    local str = nil -- Accumulates the content

    if tNeedsRepair and #tNeedsRepair > 0 then
        local strNeeds = kTextFormat.Chat.NeedsRepair.HeaderText

        for i,v in ipairs(tNeedsRepair) do
            strNeeds = strNeeds.."\n  "..v.strName
        end

        if str == nil then str = strNeeds else str = str..strNeeds end
    end

    if tUpcomingRepair and #tUpcomingRepair > 0 and self.tOptions.ShowUpcomingInChat then
        local strUpcomingRepair = kTextFormat.Chat.UpcomingRepair.HeaderText

        for i,v in ipairs(tUpcomingRepair) do 
            strUpcomingRepair = strUpcomingRepair.."\n "..v.strName.." ("..self:TimeRemainingString(v.fRemainingHours)..")"
        end

        if str == nil then str = strUpcomingRepair else str = str.."\n"..strUpcomingRepair end
    end

    return str
end

---------------------------------------------------------------------------------------------------
-- Some quick helpers to send some output
function RepairAlerts:ErrorOut(str)
    ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Debug, kAddonTitle..": "..str)
end

function RepairAlerts:MessageOut(str)
    ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Realm, str)
end

---------------------------------------------------------------------------------------------------
-- RepairAlertsConfigureForm Functions
---------------------------------------------------------------------------------------------------
function RepairAlerts:OnOK()
    -- Get the checkbox values and save them in our Options
    -- But if the bubble (or chat) is turned off, don't save an option to show optional data
    --TODO: Does it matter if we call FindChild repeatedly versus making a new local for the wnd?
    self.tOptions.ShowInBubble         = self.wndConfigure:FindChild(kControls.ShowInBubble):IsChecked()
    self.tOptions.ShowUpcomingInBubble = self.wndConfigure:FindChild(kControls.ShowUpcomingInBubble):IsChecked() and self.tOptions.ShowInBubble
    
    local Seconds = self.wndConfigure:FindChild(kControls.SliderBubbleDuration):GetValue()
    if Seconds and tonumber(Seconds) then self.tOptions.BubbleDuration = tonumber(Seconds) end

    self.tOptions.ShowInChat           = self.wndConfigure:FindChild(kControls.ShowInChat):IsChecked()
    self.tOptions.ShowUpcomingInChat   = self.wndConfigure:FindChild(kControls.ShowUpcomingInChat):IsChecked()   and self.tOptions.ShowInChat

    -- Save off the upcoming duration window
    local Hours = self.wndConfigure:FindChild(kControls.EditUpcomingDuration):GetText()
    if Hours and tonumber(Hours) then self.tOptions.UpcomingDuration = tonumber(Hours) end

    -- The checkbox for whether clicking the Alert window should open the Remodel/Repair screen
    self.tOptions.LandscapeOnBubbleClick = self.wndConfigure:FindChild(kControls.LandscapeOnBubbleClick):IsChecked()
    
    -- Now go through the ignore checkboxes and pick up the ids of any plots we ignored
    if HousingLib.IsHousingWorld() and HousingLib.IsOnMyResidence() then         
        self.tOptions.IgnoredPlots = {}
        local wndFrameIgnorePlots = self.wndConfigure:FindChild(kControls.FrameIgnorePlots)
        if wndFrameIgnorePlots == nil then return end --TODO: Find a better error response
        for i = 1, kMaxIgnore do
            local wndIgnoreCheck = wndFrameIgnorePlots:FindChild(kControls.IgnoreCheckPrefix..tostring(i))
            if wndIgnoreCheck ~= nil and wndIgnoreCheck:IsChecked() then
                self.tOptions.IgnoredPlots[wndIgnoreCheck:GetData()] = true
            end
        end
    end

    self.wndConfigure:Close() -- hide the window
end

function RepairAlerts:OnCancel()
    self.wndConfigure:Close() -- hide the window
end

---------------------------------------------------------------------------------------------------
function RepairAlerts:OnShowConfiguration(wndHandler, wndControl)
    -- Despite expectations, the ShowWindow event on a window will be called for some child controls 
    -- as well as the window itself. The wndHandler parameter will be the form for every call, but the
    -- wndControl will reference the affected control. For the first call (the one showing the whole 
    -- form), wndControl will also be the wndHandler (and the form window itself, if you've saved that)
    if wndControl ~= self.wndConfigure then return end

    -- Set the check boxes according to the current settings
    self.wndConfigure:FindChild(kControls.ShowInBubble):SetCheck(self.tOptions.ShowInBubble)
    self.wndConfigure:FindChild(kControls.ShowUpcomingInBubble):SetCheck(self.tOptions.ShowUpcomingInBubble)

    self.wndConfigure:FindChild(kControls.EditBubbleDuration):SetText(self.tOptions.BubbleDuration)
    self.wndConfigure:FindChild(kControls.SliderBubbleDuration):SetValue(self.tOptions.BubbleDuration)
    
    self.wndConfigure:FindChild(kControls.ShowInChat):SetCheck(self.tOptions.ShowInChat)
    self.wndConfigure:FindChild(kControls.ShowUpcomingInChat):SetCheck(self.tOptions.ShowUpcomingInChat)

    -- Set the duration edit box and slider
    self.wndConfigure:FindChild(kControls.EditUpcomingDuration):SetText(tostring(self.tOptions.UpcomingDuration))
    self.wndConfigure:FindChild(kControls.SliderUpcomingDuration):SetValue(self.tOptions.UpcomingDuration)

    -- Whether clicking the bubble shows the landscape and remodel UI
    self.wndConfigure:FindChild(kControls.LandscapeOnBubbleClick):SetCheck(self.tOptions.LandscapeOnBubbleClick)

    -- Fill in the ignore options with the plots on the place
    if HousingLib.IsHousingWorld() and HousingLib.IsOnMyResidence() then
        local iIgnoreCheck = 0
        local nPlots = HousingLib.GetPlotCount()
        for iPlot = 1, nPlots do
            local tPlotInfo = HousingLib.GetPlot(iPlot)

            if tPlotInfo.eUpkeepType == HousingLib.HousingUpkeepType.Timed then
                -- We have a timed upkeep plot that they may want to ignore
                iIgnoreCheck = iIgnoreCheck + 1
                if iIgnoreCheck > kMaxIgnore then return end --TODO: Handle this unlikely error better
                local wndIgnoreCheck = self.wndConfigure:FindChild(kControls.IgnoreCheckPrefix..tostring(iIgnoreCheck))
                if wndIgnoreCheck == nil then return end --TODO: Handle this more gracefully
                wndIgnoreCheck:SetCheck(false)
                if self.tOptions.IgnoredPlots[tPlotInfo.nPlugItemId] == true then wndIgnoreCheck:SetCheck(true) end
                wndIgnoreCheck:SetText(tPlotInfo.strName)
                wndIgnoreCheck:Show(true)
                wndIgnoreCheck:SetData(tPlotInfo.nPlugItemId)
            end
        end

        -- If they have fewer than 6 timed plots, hide the remaining items
        -- Start at one past where we last showed anything
        iIgnoreCheck = iIgnoreCheck + 1 
        if iIgnoreCheck < kMaxIgnore then
            for iCheckToHide = iIgnoreCheck, kMaxIgnore do
                local wndCheckToHide = self.wndConfigure:FindChild(kControls.IgnoreCheckPrefix..tostring(iCheckToHide))
                if wndCheckToHide == nil then return end --TODO Find a better error response
                wndCheckToHide:Show(false)
                wndCheckToHide:SetData(nil)
            end
        end
    else
        -- Not at our house, so we can't enumerate the plots
        for i = 1, kMaxIgnore do
            local wnd = self.wndConfigure:FindChild(kControls.IgnoreCheckPrefix..tostring(i))
            if wnd and wnd:IsValid() then
                wnd:Show(false)
                wnd:SetData(nil)
            end
        end
    end

end

-- These manage enabling/disabling the secondary options when the master option is disabled/enabled. 
-- In this case we want to turn off "Show upcoming repairs in chat" if chat output is off, for example
function RepairAlerts:OnShowAlertCheck(wndHandler, wndControl, eMouseButton)
    wndControl:GetParent():FindChild(kControls.ShowUpcomingInBubble):Enable(true)
end

function RepairAlerts:OnShowAlertUncheck(wndHandler, wndControl, eMouseButton)
    wndControl:GetParent():FindChild(kControls.ShowUpcomingInBubble):Enable(false)
end

function RepairAlerts:OnShowChatCheck(wndHandler, wndControl, eMouseButton)
    wndControl:GetParent():FindChild(kControls.ShowUpcomingInChat):Enable(true)
end

function RepairAlerts:OnShowChatUncheck(wndHandler, wndControl, eMouseButton)
    wndControl:GetParent():FindChild(kControls.ShowUpcomingInChat):Enable(false)
end

-------------------------------------------------------------------------------
-- Keep the durection slider and the duration edit box in sync
function RepairAlerts:OnUpcomingDurationSliderChanging(wndHandler, wndControl, fNewValue, fOldValue, bOkToChange)
    local Hours = math.floor(fNewValue)
    self.wndConfigure:FindChild(kControls.EditUpcomingDuration):SetText(tostring(Hours))
    self.TempUpcomingDuration = Hours
    wndControl:SetValue(Hours) -- I guess we can't just let the framework do this?
end

function RepairAlerts:OnEditUpcomingDurationReturn(wndHandler, wndControl, strText)
    local Hours = tonumber(strText)

    if   Hours == nil
      or Hours <  kDefault.UpcomingDurationMin
      or Hours >  kDefault.UpcomingDurationMax then 
        Hours = kDefault.UpcomingDuration
    end

    Hours = math.floor(Hours)
    wndControl:SetText(tostring(Hours))
    self.wndConfigure:FindChild(kControls.SliderUpcomingDuration):SetValue(Hours)
    self.TempUpcomingDuration = Hours -- We need this in case they later escape out of an edit after making a change
end

function RepairAlerts:OnRepairClickCheck(wndHandler, wndControl, eMouseButton)
    
end

function RepairAlerts:OnRepairClickUncheck(wndHandler, wndControl, eMouseButton)
end

function RepairAlerts:OnEditUpcomingDurationEscape(wndHandler, wndControl)
    -- Just reset the control
    if self.TempUpcomingDuration == nil then self.TempUpcomingDuration = self.tOptions.UpcomingDuration end
    self.wndConfigure:FindChild(kControls.SliderUpcomingDuration):SetValue(self.TempUpcomingDuration)
    wndControl:SetText(tostring(self.TempUpcomingDuration))
end

-------------------------------------------------------------------------------
-- Keep the bubble display duration slider and edit box in sync
function RepairAlerts:OnBubbleDurationSliderChanging( wndHandler, wndControl, fNewValue, fOldValue, bOkToChange )
    local Seconds = math.floor(fNewValue)
    self.wndConfigure:FindChild(kControls.EditBubbleDuration):SetText(tostring(Seconds))
    self.TempBubbleDuration = Seconds
    wndControl:SetValue(Seconds)
end

function RepairAlerts:OnEditBubbleDurationReturn(wndHandler, wndControl, strText)
    local Seconds = tonumber(strText)
    
    if   Seconds == nil 
      or Seconds <  kDefault.BubbleDurationMin 
      or Seconds >  kDefault.BubbleDurationMax then 
        Seconds = kDefault.BubbleDuration 
    end 

    Seconds = math.floor(Seconds)
    self.wndConfigure:FindChild(kControls.SliderBubbleDuration):SetValue(Seconds)
    self.TempBubbleDuration = Seconds -- Store this in case they change it and then escape
    wndControl:SetText(tostring(Seconds))
end

function RepairAlerts:OnEditBubbleDurationEscape(wndHandler, wndControl)
    if self.TempBubbleDuration == nil then self.TempBubbleDuration = self.tOptions.BubbleDuration end
    self.wndConfigure:FindChild(kControls.SliderBubbleDuration):SetValue(self.TempBubbleDuration)
    wndControl:SetText(tostring(self.TempBubbleDuration))
end

-----------------------------------------------------------------------------------------------
-- RepairAlerts Instance
-----------------------------------------------------------------------------------------------
local RepairAlertsInst = RepairAlerts:new()
RepairAlertsInst:Init()
